@@ -1,12 +1,12 @@
 ;;; sxhkdrc-mode.el --- Major mode for sxhkdrc files (Simple X Hot Key Daemon) -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022  Free Software Foundation, Inc.
+;; Copyright (C) 2022-2023  Free Software Foundation, Inc.
 
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
 ;; Maintainer: Protesilaos Stavrou General Issues <~protesilaos/general-issues@lists.sr.ht>
 ;; URL: https://git.sr.ht/~protesilaos/sxhkdrc-mode
 ;; Mailing-List: https://lists.sr.ht/~protesilaos/general-issues
-;; Version: 0.1.4
+;; Version: 1.0.0
 ;; Package-Requires: ((emacs "27.1"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -51,8 +51,11 @@ Space Partitioning Window Manager (BSPWM)."
   '((key-modifier . ( "control" "ctrl" "shift" "alt" "meta" "super" "hyper"
                       "mod1" "mod2" "mod3" "mod4" "mod5"))
     (key-generic . "^\\({.*?}\\|\\<.*?\\>\\)")
+    (key-line . "^\\({.*?}\\|\\<.*?\\>\\).*$")
+    (outline . "^###+ ")
     (comment . "^\\([\s\t]+\\)?#.*$")
     (command . "^[\s\t]+\\([;]\\)?\\(\\_<.*?\\_>\\)")
+    (command-line . "^[\s\t]+\\([;]\\)?\\(\\_<.*?\\_>\\).*$")
     (indent-other . 0)
     (indent-command . 4))
   "List of associations for sxhkdrc syntax.")
@@ -101,27 +104,62 @@ key chord chain (demarcated by a colon or semicolon)."
 (defun sxhkdrc-mode-indent-line ()
   "Indent line according to `sxhkdrc-mode-syntax'."
   (interactive)
-  (let ((syntax sxhkdrc-mode-syntax))
+  (let* ((syntax sxhkdrc-mode-syntax)
+         (command (alist-get 'command syntax))
+         (key (alist-get 'key-generic syntax))
+         (indent-other (alist-get 'indent-other syntax))
+         (indent-command (alist-get 'indent-command syntax))
+         indent)
+    ;; The `or' statements here are needed because this will work with
+    ;; `electric-indent-mode' that does RET+TAB in one go.
     (save-excursion
-      (beginning-of-line)
-      (delete-horizontal-space)
+      (goto-char (line-beginning-position))
+      (skip-chars-forward "\t " (line-end-position))
       (cond
-       ((looking-at (alist-get 'comment syntax))
-        (indent-to (alist-get 'indent-other syntax)))
-       ((or (not (looking-at (alist-get 'key-generic syntax)))
-            (save-excursion
+       ;; If the command continues to a new line by virtue of a
+       ;; trailing \ then we indent accordingly.
+       ((or (looking-at command)
+            (progn
               (forward-line -1)
               (beginning-of-line)
-              (looking-at (alist-get 'key-generic syntax))))
-        (indent-to (alist-get 'indent-command syntax)))))))
+              (or (re-search-forward ".*\\\\$" (line-end-position) t)
+                  (looking-at key))))
+        (setq indent indent-command))
+       ;; If the previous line is a command that does not end with a
+       ;; backslash, we want to reset indentation.
+       ((or (looking-at command)
+            (progn
+              (forward-line -1)
+              (beginning-of-line)
+              (looking-at command)))
+        (setq indent indent-other))
+       ;; If we are on a key definition, the following will be a
+       ;; command.
+       ((or (looking-at key)
+            (progn
+              (forward-line -1)
+              (beginning-of-line)
+              (looking-at key)))
+        (setq indent indent-command))))
+    (if indent
+        (progn
+          (delete-horizontal-space)
+          (indent-to indent))
+      'no-indent)))
+
+(defvar sxhkdrc-mode-map (make-sparse-keymap)
+  "Local keymap for `sxhkdrc-mode' buffers.")
 
 ;;;###autoload
-(define-derived-mode sxhkdrc-mode prog-mode "SXHKDRC"
+(define-derived-mode sxhkdrc-mode fundamental-mode "SXHKDRC"
   "Major mode for editing sxhkdrc files (Simple X Hot Key Daemon)."
   (setq-local indent-line-function 'sxhkdrc-mode-indent-line
-              comment-start "# "
-              comment-start-skip "#+ *")
-  (setq font-lock-defaults '(sxhkdrc-mode-font-lock-keywords)))
+              comment-start "#"
+              comment-start-skip (concat (regexp-quote comment-start) "+\\s *")
+              outline-regexp (alist-get 'outline sxhkdrc-mode-syntax)
+              imenu-generic-expression `(("Command" ,(alist-get 'command-line sxhkdrc-mode-syntax) 0)
+                                         ("Key" ,(alist-get 'key-line sxhkdrc-mode-syntax) 0))
+              font-lock-defaults '(sxhkdrc-mode-font-lock-keywords)))
 
 ;;;###autoload
 (add-to-list 'auto-mode-alist '("sxhkdrc\\'" . sxhkdrc-mode))
